@@ -13,7 +13,7 @@ feature propper object orientation like munin-update and will have to
 wait until later.
 
 Copyright (C) 2002-2010 Jimmy Olsen, Audun Ytterdal, Kjell Magne
-ierud, Nicolai Langfeldt, Linpro AS, Redpill Linpro AS and others.
+Øierud, Nicolai Langfeldt, Linpro AS, Redpill Linpro AS and others.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -84,6 +84,9 @@ if ($RRDs::VERSION >= 1.3) {
     $AddNAN = 'ADDNAN';
 }
 
+# the ":dashes" syntax for LINEs is supported since rrdtool 1.5.3
+my $RRDLineThresholdAttribute = ($RRDs::VERSION < 1.50003) ? '' : ':dashes';
+
 # Force drawing of "graph no".
 my $force_graphing = 0;
 my $force_lazy     = 1;
@@ -185,7 +188,7 @@ my @init_limit_hosts = @limit_hosts;
 my @init_limit_services = @limit_services;
 
 sub process_pinpoint {
-    my ($pinpoint, $arg_name, $arg_value) = @_;     
+    my ($pinpoint, $arg_name, $arg_value) = @_;
     # XXX - Special hack^h^h^h^h treatment for --pinpoint
     if ($arg_value && $arg_value =~ m/^(\d+),(\d+)$/ ) {
 	# "pinpoint" replaces all the other timing options
@@ -319,13 +322,13 @@ sub graph_startup {
 
     $max_running = &munin_get($config, "max_graph_jobs", $max_running);
 
-    if ($config->{"rrdcached_socket"}) { 
+    if ($config->{"rrdcached_socket"}) {
 	    if ($RRDs::VERSION >= 1.3){
 		# Using the RRDCACHED_ADDRESS environnement variable, as
                 # it is way less intrusive than the command line args.
                 $ENV{RRDCACHED_ADDRESS} = $config->{"rrdcached_socket"};
-	    } else { 
-		    ERROR "[ERROR] RRDCached feature ignored: RRD version must be at least 1.3. Version found: " . $RRDs::VERSION; 
+	    } else {
+		    ERROR "[ERROR] RRDCached feature ignored: RRD version must be at least 1.3. Version found: " . $RRDs::VERSION;
 	    }
     }
 
@@ -367,7 +370,7 @@ sub graph_main {
     # The loaded $config is stale within 5 minutes.
     # So, we need to reread it when this happens.
     $config = munin_readconfig_part('datafile');
-   
+
     # Reset an eventual custom size
     $size_x 	    = undef;
     $size_y         = undef;
@@ -585,7 +588,7 @@ sub expand_specials {
     # Test if already expanded
     {
         my $cached = $service->{"#%#expand_specials"};
-	if (defined $cached) { 
+	if (defined $cached) {
 		DEBUG "[DEBUG] expand_specials(): already processed " . munin_dumpconfig_as_str($cached);
         	return $cached;
 	}
@@ -1111,6 +1114,7 @@ sub process_service {
 	
 	# Select a default colour if no explict one
 	$colour ||= ($single_value) ? $single_colour : $COLOUR[$field_count % @COLOUR];
+        my $warn_colour = $single_value ? "ff0000" : $colour;
 
         # colour needed for transparent predictions and trends
         munin_set($field, "colour", $colour);
@@ -1171,24 +1175,10 @@ sub process_service {
                 my ($warn_min, $warn_max) = split(':', $tmpwarn,2);
 
                 if (defined($warn_min) and $warn_min ne '') {
-                    unshift(
-                        @rrd,
-                        "HRULE:" 
-                            . $warn_min
-                            . "#" . (
-                            $single_value
-                            ? "ff0000"
-                            : $COLOUR[($field_count - 1) % @COLOUR]));
+                    unshift(@rrd, "HRULE:${warn_min}#${warn_colour}${RRDLineThresholdAttribute}");
                 }
                 if (defined($warn_max) and $warn_max ne '') {
-                    unshift(
-                        @rrd,
-                        "HRULE:" 
-                            . $warn_max
-                            . "#" . (
-                            $single_value
-                            ? "ff0000"
-                            : $COLOUR[($field_count - 1) % @COLOUR]));
+                    unshift(@rrd, "HRULE:${warn_max}#${warn_colour}${RRDLineThresholdAttribute}");
                 }
             }
 
@@ -1234,7 +1224,7 @@ sub process_service {
             $label =~ s/:/\\:/g if defined $label;
             unshift(
                 @rrd,
-                "HRULE:" 
+                "HRULE:"
                     . $number
                     . (
                     $ldcolour ? "#$ldcolour"
@@ -1249,24 +1239,10 @@ sub process_service {
             my ($warn_min, $warn_max) = split(':', $tmpwarn,2);
 
             if (defined($warn_min) and $warn_min ne '') {
-                unshift(
-                    @rrd,
-                    "HRULE:" 
-                        . $warn_min
-                        . "#" . (
-                        $single_value
-                        ? "ff0000"
-                        : $COLOUR[($field_count - 1) % @COLOUR]));
+                unshift(@rrd, "HRULE:${warn_min}#${warn_colour}${RRDLineThresholdAttribute}");
             }
             if (defined($warn_max) and $warn_max ne '') {
-                unshift(
-                    @rrd,
-                    "HRULE:" 
-                        . $warn_max
-                        . "#" . (
-                        $single_value
-                        ? "ff0000"
-                        : $COLOUR[($field_count - 1) % @COLOUR]));
+                unshift(@rrd, "HRULE:${warn_max}#${warn_colour}${RRDLineThresholdAttribute}");
             }
         }
     }
@@ -1609,7 +1585,7 @@ sub handle_trends {
         my $fieldname = munin_get_node_name($field);
 	my $colour = $single_colour;
 
-	# Skip virtual fieldnames, otherwise beware of $hash->{foo}{bar}. 
+	# Skip virtual fieldnames, otherwise beware of $hash->{foo}{bar}.
 	#
 	# On a sidenote, what's the output of the following code ?
 	# perl -e '$a = {}; if ($a->{foo}{bar}) { print "Found Foo/Bar\n"; } \
@@ -1618,8 +1594,8 @@ sub handle_trends {
 
         if (defined $service->{$fieldname}{'colour'}) {
             $colour = "$service->{$fieldname}{'colour'}66";
-        }                                                                                                                                                    
-                                                                                                                                                             
+        }
+
         my $rrd_fieldname = get_field_name($fieldname);
 
         my $cdef = "";
@@ -1649,7 +1625,7 @@ sub handle_trends {
 
     push(@complete,
         "COMMENT:Last update$RRDkludge: "
-        . RRDescape(scalar localtime($lastupdate))                                                                                                   
+        . RRDescape(scalar localtime($lastupdate))
         . "\\r");
 
     # If pinpointing, --end should *NOT* be changed
@@ -1793,9 +1769,9 @@ sub expand_cdef {
         my $fieldname = munin_get_node_name($field);
         my $rrdname = &orig_to_cdef($service, $fieldname);
         if ($cdef =~ /\b$fieldname\b/) {
-            $max =~ s/([,=])$fieldname([,=]|$)/$1a$rrdname$2/g;
-            $min =~ s/([,=])$fieldname([,=]|$)/$1i$rrdname$2/g;
-            $avg =~ s/([,=])$fieldname([,=]|$)/$1g$rrdname$2/g;
+            $max =~ s/(?<=[,=(])$fieldname(?=[,=)]|$)/a$rrdname/g;
+            $min =~ s/(?<=[,=(])$fieldname(?=[,=)]|$)/i$rrdname/g;
+            $avg =~ s/(?<=[,=(])$fieldname(?=[,=)]|$)/g$rrdname/g;
         }
     }
 
