@@ -81,7 +81,8 @@ sub _read_old_service_configs {
 	    return {};
 	}
 	eval {
-	    $oldconfig->parse_config($file);
+            # parse file but skip malformed lines
+            $oldconfig->parse_config($file, 1);
 	};
 	if ($EVAL_ERROR) {
 	    WARN "[Warning] Could not parse datafile $datafile: $EVAL_ERROR";
@@ -359,8 +360,12 @@ sub _print_old_service_configs_for_failed_workers {
 	    next if ($datum eq 'group')
 		or ($datum eq 'host_name');
 
-	    printf $handle "%s:%s %s\n", $worker, $datum, $workerdata->{$datum};
-	    munin_set_var_path($datafile_hash, $worker . ":". $datum, $workerdata->{$datum});
+	    if (defined $workerdata->{$datum}) {
+		printf $handle "%s:%s %s\n", $worker, $datum, $workerdata->{$datum};
+		munin_set_var_path($datafile_hash, $worker . ":". $datum, $workerdata->{$datum});
+	    } else {
+		WARN "[Warning] no data $worker -> $datum";
+	    }
 	}
 	
     }
