@@ -156,9 +156,9 @@ use Munin::Plugin;
  base           Used for graph_args --base. Default is 1000, set to 1024 when
                 returning sizes in Kb for example.
  wildcardfilter The SQL to substitute for when a wildcard plugin is run against
-                a specific entity, for example a database. All occurrances of
+                a specific entity, for example a database. All occurrences of
                 the string %%FILTER%% will be replaced with this string, and
-                for each occurance a parameter with the value of the filtering
+                for each occurrence a parameter with the value of the filtering
                 condition will be added to the DBI statement.
  paramdatabase  Makes the plugin connect to the database in the first parameter
                 (wildcard plugins only) instead of 'template1'.
@@ -425,11 +425,15 @@ sub _connect() {
         # variables.
         my $dbname = "template1";
         $dbname = $self->{defaultdb}           if ($self->{defaultdb});
-        $dbname = $self->wildcard_parameter(0) if ($self->{paramdatabase} && !defined($nowildcard));
+        $dbname = $self->wildcard_parameter(0) if ($self->{paramdatabase} && !defined($nowildcard) && $self->wildcard_parameter(0));
         $dbname = $ENV{"PGDATABASE"}           if ($ENV{"PGDATABASE"});
-        $self->{dbh} = DBI->connect("DBI:Pg:dbname=$dbname", '', '', {pg_server_prepare => 0});
+        $self->{dbh} = DBI->connect("DBI:Pg:dbname=$dbname", '', '', {pg_server_prepare => 0, PrintError => 0});
         unless ($self->{dbh}) {
-            $self->{connecterror} = "$DBI::errstr";
+            my $err_str = "$DBI::errstr";
+            $err_str =~ s/[\r\n\t]/ /g;
+            $err_str =~ s/\h+/ /g;
+            $err_str =~ s/ $//;
+            $self->{connecterror} = $err_str;
             return 0;
         }
     }
@@ -479,7 +483,9 @@ sub get_version {
     my $v = $r->[0]->[0];
     die "Unable to detect PostgreSQL version\n"
         unless ($v =~ /^(\d+)\.(\d+).*\b/);
-    $self->{detected_version} = "$1.$2";
+    # from PostgreSQL 10 on only the major version is needed
+    # see https://www.postgresql.org/support/versioning/
+    $self->{detected_version} = ($1 >= 10) ? "$1" : "$1.$2";
 }
 
 sub get_versioned_query {
